#include <iostream>
#include <string>
#include <cctype>	// pour isdigit

using namespace std;

/* Remarque: dans toute cette correction les tests repetitifs
 * ('0' <= c and c <= '9') peuvent etre avantageusement remplaces par
 * isdigit(c) en incluant en tete de fichier
 * #include <cctype>
 */

/*
 * Prend en paramètre une string représentant une chaine compressee et
 * renvoie la longeur de sa version decompressee.
 */
size_t longueur(string s) {
  size_t i = 0, l = s.size(); int somme = 0;
  while(i < l) {
    // Forcement une lettre, eventuellement suivie d'un facteur de repetition.
    // Dans ce cas on traite les deux carcateres d'un coup.
    if (i+1 < l and isdigit(s.at(i+1))) {
      somme = somme + s.at(i+1) - '0'; i = i+2;
    } else { somme = somme + 1; i = i+1; }
  }
  return somme;
}

/* renvoie la version decompressee d'une chaine compressee */
string decompresse(string t1) {
  string t2;	// par defaut, initialise à la chaine vide.
  size_t i = 0, l1 = t1.size();
  while(i < l1) {
    char c1 = t1[i];
    t2.push_back(c1);	 // on ecrit toujours au moins une occurrence de c
    i = i+1;
    if (i < l1) { // verifier s'il s'agit d'un facteur de repetition
      char c2 = t1.at(i);
      if (isdigit(c2)) { // oui, on decompresse
	for(int k = c2 - '0'; k > 1; k--) { t2.push_back(c1); }
	i = i+1;
      }
    } // dans les deux autres cas: rien a faire !
  }
  return t2;
}


/* renvoie la forme compressee d'une chaine */
string compresse(string t1) {
  size_t i = 0, l1 = t1.size();
  string t2;

  while (i < l1) {
    char c = t1.at(i); int nb = 1;
    i = i+1;
    while (nb < 9 and i < l1 and t1.at(i) == c) { i = i+1; nb = nb+1; }
    if (nb > 2) {
      t2.push_back(c); t2.push_back('0'+ nb);
    } else { while(nb > 0) { t2.push_back(c); nb = nb-1; } }
  }
  return t2;
}


/*
 * Verifie si les versions decompressees de s1 et s2 seraient egales,
 * mais sans decompresser les chaines.
 * Ne suppose pas que s1 et s2 ont ete comopressees par le meme algo, juste
 * qu'elles verifient le format indique. Les sequences d'un meme caractere
 * ont donc pu etre compressees differement dans s1 et s2.
 */
bool egale(string s1, string s2) {
  size_t i1 = 0, i2 = 0; // indices dans chacune des deux chaines.
  char c1, c2;           // caractere courant dans chaque chaine
  int n1 = 0, n2 = 0;    // nombre courant d'occurrences de ce caractere

  while (i1 < s1.size() or i2 < s2.size()) {

    // On s'arrange pour avoir dans n1 et n2 le nombre d'occurrences du
    // caractere courant qu'on avait dans s1 et s2.
    // si n1 (resp. n2) est nul, il faut progresser dans s1 (resp. s2) en lisant
    // un nouveau caractere et son eventuel nombre d'occurrences.
    // L'initialisation de n1 et n2 a 0 ci-dessus force une premiere lecture
    // dans les deux chaines. Les valeurs initiales de c1 et c2 n'importent pas.
    // Quand on a obtenu les valeurs courantes de c1, n1 et c2, n2, il n'y a
    // plus qu'à comparer.

    if (n1 == 0) {
      // s1 est epuise mais pas s2 sinon on serait sorti
      if (i1 == s1.size()) { return false; }
      c1 = s1.at(i1); i1 = i1+1;
      // Recalcule la nouvelle valeur pour n1: combien d'occurrences successives
      // de la nouvelle valeur de c1 a-t-on ?
      if (i1 < s1.size() and s1.at(i1) >= '0' and s1.at(i1) <= '9') {
	n1 = s1.at(i1) - '0'; i1 = i1+1;
      } else { n1 = 1; }
    }
    if (n2 == 0) { // le symetrique
      if (i2 == s2.size()) { return false; }
      c2 = s2.at(i2); i2 = i2+1;
      if (i2 < s2.size() and s2.at(i2) >= '0' and s2.at(i2) <= '9') {
	n2 = s2.at(i2) - '0'; i2 = i2+1;
      } else { n2 = 1; }
    }
    if (c1 != c2) { return false; }
    else if (n1 < n2) { n2 = n2 - n1; n1 = 0; }
    else { n1 = n1 - n2; n2 = 0;}
  }
  // Il peut rester des caracteres definis implicitement par n1 et n2
  return n1 == n2;  
}


int main() {
  string t1 = "aaaabbcdddddddddddddeee";
  string t2(t1); // cree une copie de t1
  string vide = "";
  string res, resVide, res2Vide;

  t2 = compresse(t1);
  res = decompresse(t2);
  cout << "Compression de " << t1 << ", longueur: " << t1.size() << " = "
       << t2 << ", longueur: " << t2.size() << endl;
  cout << "Decompression de " << t2 << " = " << res << ", longueur: "
       << res.size() << endl;

  resVide = compresse(vide);
  res2Vide = decompresse(resVide);
  cout << "Compression de: " << vide << " = " << resVide << ", longueur: "
       << longueur(vide) << endl;
  cout << "Decompression de " << resVide << " = " << res2Vide
       << ", longueur " << res2Vide.size() << endl;

  if (t1 != res or  vide != res2Vide) {
    printf("Erreur: Chaines differentes apres compression/decompression !\n");
  }


  if (! egale(t1, t1)) { cout << "pb avec l'egalite de t1\n"; }
  else  { cout << "OK avec l'egalite de t1\n"; }
  string x1 = "a4b2cd9d4e3";
  string x2 = "aaaabbcd7d6e2e";
  string x3 = "aaaabbcd5d6e2e";
  string x4 = "aaaabbcd5d6e2";
  string x5= "aaaabbcd7d6e2ef";
  string x6 = "a7";
  string x7 = "a9";

  if (! egale(x1, x2)) { cout << "pb avec l'egalite de x1, x2\n"; }
  else { cout << "OK avec l'egalite de x1, x2\n"; }

  if (! egale(x1, x2)) { cout << "pb avec l'egalite de x1, x2\n"; }
  else { cout << "OK avec l'egalite de x1, x2\n"; }

  if (egale(x1, x3)) { cout << "pb avec l'egalite de x1, x2\n"; }
  else { cout << "OK avec l'egalite de x1, x3\n"; }

  if (egale(x2, x4)) { cout << "pb avec l'egalite de x2, x4\n"; }
  else { cout << "OK avec l'egalite de x2, x4\n"; }

  if (egale(x2, x5)) { cout << "pb avec l'egalite de x2, x5\n"; }
  else { cout << "OK avec l'egalite de x2, x5\n"; }

  if (egale(x6, x7)) { cout << "pb avec l'egalite de x6, x7\n"; }
  else { cout << "OK avec l'egalite de x6, x7\n"; }

  return 0;
}
