#include <iostream>
#include <vector>

using namespace std;

/* Dans les exercices associes, verifier que le code fonctionne quand
 * l'un des deux, ou les deux, tableaux sont vides.
 *
 * L'interet des deux versions est de voir la structure du code selon
 * que la premiere boucle est controlee par un 'and' ou par un 'or'.
 * Les faire raisonner sur la condition vraie a l'interieur de la boucle
 * et ce qu'on peut en deduire quant a la validite des indices.
 * Idem pour la condition apres la sortie de boucle.
 * Leur faire faire les deux versions.
 */

vector<int> fusion(vector<int> t1, vector<int> t2) {
  size_t n1 = t1.size(), n2 = t2.size(), i = 0, j = 0;
  vector<int> t3;

  t3.reserve(n1+n2);	// facultatif
  // Si les deux tableaux sont initialement vides, on ne passera dans
  // aucune des 3 boucles.
  // Dans tous les autres cas, la premiere boucle va forcement finir un
  // tableau avant l'autre. On passera forcement par une et une seule
  // des deux dernieres boucles
  while (i < n1 and j < n2) {
    // puisqu'aucun des deux tableaux n'est epuise, on peut comparer
    if (t1.at(i) < t2.at(j)) { t3.push_back(t1.at(i)); i = i+1; }
    else { t3.push_back(t2.at(j)); j = j+1; }
  }
  // epuiser l'unique tableau non vide (si les deux tabelaux n'etaient
  // pas initialement vides
  while (i < n1) { t3.push_back(t1.at(i)); i = i+1; }
  while (j < n2) { t3.push_back(t2.at(j)); j = j+1; }
  return t3;
}


/* version alternative: le code de la premier boucle devient plus complique
 * car l'un des deux tableaux peut etre epuise. Par contre en sortie de cette
 * boucle, il n'y a plus rien a faire.
 */
vector<int> fusionV2(vector<int> t1, vector<int> t2) {
  size_t n1 = t1.size(), n2 = t2.size(), i = 0, j = 0;
  vector<int> t3;

  t3.reserve(n1+n2);	// facultatif
  while (i < n1 or j < n2) {
    // Les faire reflechir: dans quelles conditions peut-on etre sur
    // qu'un acces a un des tableaux va etre correct ?
    if (j == n2 or (i < n1 and t1.at(i) < t2.at(j))) {
      // si j == n2, forcement i < n1 et on peut copier l'element de t1
      // sinon on ne peut rien dire sur le fait que i1 soit un indice correct.
      // Il ne faut copier a partir de t1 que si i < n1 et l'element associe est
      // plus petit (ou plus petit ou egal, peu importe). Sinon on doit copier
      // l'element de t2.
      t3.push_back(t1.at(i)); i = i+1;
    } else { t3.push_back(t2.at(j)); j = j+1; }
  }
  return t3;
}


void imprimer(vector<int> t, string nom) {
  cout << nom << ": [ ";
  for(size_t i =  0; i < t.size(); i = i+1) cout << " " << t.at(i);
  cout << "]\n";
}


int main() {
  vector<int> t1 = { 1, 12, 33, 45 , 62 };
  vector<int> t2 = { 0, 33, 55, 62, 67, 75, 100 };
  vector<int> res, res2;

  res = fusion(t1, t2);
  res2 = fusionV2(t1, t2);


  imprimer(t1, "t1");
  imprimer(t2, "t2");

  imprimer(res, "Résultat avec fusion");
  imprimer(res2, "Résultat avec fusionV2");  

  // on modifie t1 pour que la fusion se temine par un element de t1
  t1.push_back(172);
  res2 = fusionV2(t1, t2);
  imprimer(res2, "Resultat avec fusionV2 et t1 modifie");

  return 0;
}
