#include <iostream>
#include <string>
#include <climits>	/* Pour INT_MAX */

using namespace std;

/* Question: fait-on une version qui prend une reference sur une string,
 * ou une version qui renvoie une string en resultat ?
 */

/* Stocke dans t la representation textuelle de val.
 * On suppose que les entiers sont positifs ou nuls (pour des negatifs, il
 * ne suffit pas de prendre la valeur absolue, puisque en complement a 2,
 * l'oppose de INT_MIN n'est pas representable).
 * Ca compliquerait inutilement le code pour ce qui nous interesse.
 *
 * Version1 : on ecrit le resultat a l'envers, puis on le retourne...
 */
void int2string(int val, string &t){
  int v; size_t i, j;
  t.clear();	// reinitialiser t, au cas ou...
  if (val == 0) { t.push_back('0'); return; }
  // on obtient les chiffres de la representation par divisions successives.
  // Le chiffre courant est le reste de la division par 10.
  v = val;
  while (v > 0) {
    t.push_back((v % 10) + '0'); /* passage de l'entier au caractere */
    v = v / 10;
  }
  // il ne reste plus qu'a echanger les caractere le plus a gauche avec celui
  // le plus a droite et a recommencer jusqu'au milieu de la chaine.
  // La plus petite chaine est de taille 1 (donc i et j valent 0) et ce n'est
  // pas la peine de la retourner. Sinon i vaut au moins 1 et on ne le
  // decrementera jamais en-dessous de 0 avant d'avoir croise j. Donc pas de
  // probleme avec i = i-1 meme si i est de type size_t donc "unsigned".
  j = 0; i = t.size()-1;
  while (j < i) {
    char c = t.at(i);
    t.at(i) = t.at(j); t.at(j) = c;
    i = i-1; j = j+1;
  }
}

/* Deuxieme version: on ecrit directement les caracteres a la bonne place.
 * On suppose toujours que l'entier passe en parametre est positif ou nul.
 * On ecrit d'abord le coefficient de la puissance la plus grande, qu'on
 * cherche donc.
 *
 * Version incorrecte: chercher la 1ere puissance de 10 plus grande que val.
 * Le probleme est qu'on risque de sortir de la plage de valeurs correcte
 * pour le type de val. Si on essaye de prendre un type plus grand, on n'a pas
 * de garantie qu'il offre plus de valeurs (depend de l'implementation; la norme
 * demande juste que le type 'au-dessus' dans la hierarchie n'ait pas une plage
 * plus reduite que la plage du type 'en-dessous.
 * Par exemple, d'apres la norme
 * short: "not smaller than a char. At least 16 bits"
 * int :  "not smaller than a short. At least 16 bits"
 * long: "not smaller than an int. At least 32 bits"
 *
 * Un short et un int pourraient par exemple avoir les memes plages de valeur.
 */

void int2stringV2(int val, string &t) {
  int v;
  int p; // On ne depassera pas val, qui est suppose est un int correct
  t.clear();
  if (val == 0) { t.push_back('0'); return ;}
  p = 1; v = val;
  // cherche la puissance de 10 la plus elevee, sans depasser val.
  while (val / p >= 10) { p = p * 10; }
  while (p > 0) {
    int k = v / p;
    // Le chiffre qui nous interesse est le quotient de la division et on prend
    // le reste de la division pour poursuivre le calcul
    t.push_back(k + '0'); // passage de l'entier au  caractere
    v = v % p;
    p = p / 10; // considerer la puissance inferieure
  }
}

int main() {
 /* Sur ma machine un int prend 4 octets...
  * Sur quatre octets, la plus grande valeur sera 2^31-1 = 2147483647.
  * Au retour, la chaine sera de taille au plus 10
  */
  int ex[12] = { 12, 1, 12385, 0, 10, 307, 327, 100, 101, 99, 7, INT_MAX};
  string res;
  int i;

  /* On pourrait eventuellement faire
   * res.reserve(10);
   * pour garantir des le depart une capacite suffisante (et raisonnable)
   * pour la string
   */

  printf("Premiere version de int2string\n");
  for(i = 0; i < 12; i = i+1) {
    int2string(ex[i], res);
    cout << "La representation de " << ex[i]
	 << " est \"" << res << "\"" << endl;
  }

  printf("\nSeconde version de int2string\n");
  for(i = 0; i < 12; i = i+1) {
    int2stringV2(ex[i], res);
    cout << "La representation de " << ex[i]
	 << " est \"" << res << "\"" << endl;
  }

  cout << "Taille finale de la string: " << res.size() << endl;

  return 0;
}
